@@ -1,5 +1,5 @@
 /*
- * $Id: BugzillaSender.h 2714 2009-06-25 14:02:27Z karijes $
+ * $Id: BugzillaSender.h 3445 2012-11-07 12:49:43Z karijes $
  *
  * ede-bug-report, a tool to report bugs on EDE bugzilla instance
  * Part of Equinox Desktop Environment (EDE).
@@ -13,6 +13,8 @@
 #ifndef __BUGZILLASENDER_H__
 #define __BUGZILLASENDER_H__
 
-bool bugzilla_send_with_progress(const char *title, const char *content);
+bool bugzilla_send_with_progress(const char *title,
+								 const char *content,
+								 const char *cc);
 
 #endif
