@@ -1,5 +1,5 @@
 /*
- * $Id: GdbOutput.h 3110 2011-10-24 08:15:47Z karijes $
+ * $Id: GdbOutput.h 3435 2012-10-23 10:20:21Z karijes $
  *
  * ede-crasher, a crash handler tool
  * Part of Equinox Desktop Environment (EDE).
@@ -13,6 +13,6 @@
 #ifndef __GDBOUTPUT_H__
 #define __GDBOUTPUT_H__
 
-bool gdb_output_generate(const char *path, edelib::TempFile &t);
+bool gdb_output_generate(const char *path, edelib::TempFile &t, int pid = -1);
 
 #endif
