@@ -1,5 +1,5 @@
 /*
- * $Id: Xsm.h 2969 2009-12-03 16:56:20Z karijes $
+ * $Id: Xsm.h 3416 2012-09-18 17:49:48Z karijes $
  *
  * Evoke, head honcho of everything
  * Part of Equinox Desktop Environment (EDE).
@@ -14,15 +14,20 @@
 #define __XSM_H__
 
 #include <edelib/XSettingsManager.h>
-#include <edelib/EdbusConnection.h>
 
+#ifdef EDELIB_HAVE_DBUS
+# include <edelib/EdbusConnection.h>
 EDELIB_NS_USING(EdbusConnection)
+#endif
+
 EDELIB_NS_USING(XSettingsData)
 
 /* XSETTINGS manager with serialization capability. Also it will write/undo to xrdb (X Resource database). */
 class Xsm : public edelib::XSettingsManager {
 private:
+#ifdef EDELIB_HAVE_DBUS
 	EdbusConnection* dbus_conn;
+#endif
 
 	/* replace XResource values from one from XSETTINGS */
 	void xresource_replace(void);
@@ -33,11 +38,14 @@ private:
 	/* serve XSETTINGS via D-Bus */
 	void xsettings_dbus_serve(void);
 public:
-	Xsm() : dbus_conn(NULL) { }
-	~Xsm();
+#ifdef EDELIB_HAVE_DBUS
+	Xsm() { dbus_conn = NULL; }
+
+	~Xsm() { delete dbus_conn; }
 
 	/* return loaded D-Bus connection */
 	EdbusConnection* get_dbus_connection(void) { return dbus_conn; }
+#endif
 
 	/* access to manager content */
 	XSettingsData* get_manager_data(void) { return manager_data; }
